/**
 * Copyright (c) 2021, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { AccessControlConstants, Show } from "@wso2is/access-control";
import { resolveUserDisplayName } from "@wso2is/core/helpers";
import { ProfileInfoInterface, TestableComponentInterface } from "@wso2is/core/models";
import {
    GenericIcon, Heading,
    LinkButton,
    PageLayout,
    Text
} from "@wso2is/react-components";
import React, { FunctionComponent, ReactElement, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useDispatch, useSelector } from "react-redux";
import { Card, Divider, Grid, Label, Placeholder } from "semantic-ui-react";
import { getGettingStartedCardIllustrations } from "./configs";
import { ApplicationListInterface, getApplicationList } from "../../../features/applications";
import {
    AppConstants,
    AppState,
    ConfigReducerStateInterface,
    EventPublisher,
    history,
    setActiveView
} from "../../../features/core";
import { UserListInterface, getUsersList } from "../../../features/users";
import { CONSUMER_USERSTORE } from "../../../features/userstores";
import { ReactComponent as SupportHeadsetIcon } from "../../assets/images/icons/support-headset-icon.svg";
import { AppViewExtensionTypes } from "../../configs";

/**
 * Proptypes for the overview page component.
 */
type GettingStartedPageInterface = TestableComponentInterface;

/**
 * Overview page.
 *
 * @param {GettingStartedPageInterface} props - Props injected to the component.
 *
 * @return {React.ReactElement}
 */
const GettingStartedPage: FunctionComponent<GettingStartedPageInterface> = (
    props: GettingStartedPageInterface
): ReactElement => {

    const {
        ["data-testid"]: testId
    } = props;

    const { t } = useTranslation();

    const dispatch = useDispatch();

    const profileInfo: ProfileInfoInterface = useSelector((state: AppState) => state.profile.profileInfo);
    const isProfileInfoLoading: boolean = useSelector((state: AppState) => state.loaders.isProfileInfoRequestLoading);
    const activeView: string = useSelector((state: AppState) => state.global.activeView);
    const docSiteURL: string = useSelector((state: AppState) => state.config.deployment.docSiteURL);
    const config: ConfigReducerStateInterface = useSelector((state: AppState) => state.config);

    const [ isConsumersAvailable, setIsConsumersAvailable ] = useState<boolean>(undefined);
    const [ isApplicationsAvailable, setIsApplicationsAvailable ] = useState<boolean>(undefined);
    const [
        isConsumerUsersFetchRequestLoading,
        setIsConsumerUsersFetchRequestLoading
    ] = useState<boolean>(undefined);
    const [
        isApplicationsFetchRequestLoading,
        setIsApplicationsFetchRequestLoading
    ] = useState<boolean>(undefined);

    const eventPublisher: EventPublisher = EventPublisher.getInstance();

    /**
     * Make sure `QUICKSTART` tab is highlighed when this page is in use.
     */
    useEffect(() => {

        if (activeView === AppViewExtensionTypes.QUICKSTART) {
            return;
        }

        dispatch(setActiveView(AppViewExtensionTypes.QUICKSTART));
    }, []);

    useEffect(() => {

        setIsConsumerUsersFetchRequestLoading(true);

        getUsersList(null, null, null, "id", CONSUMER_USERSTORE)
            .then((response: UserListInterface) => {
                setIsConsumersAvailable(response.totalResults > 0);
            })
            .catch(() => {
                // Add debug logs here one a logger is added.
                // Tracked here https://github.com/wso2/product-is/issues/11650.
            })
            .finally(() => {
                setIsConsumerUsersFetchRequestLoading(false);
            });
    }, []);

    useEffect(() => {

        setIsApplicationsFetchRequestLoading(true);

        getApplicationList(null, null, null)
            .then((response: ApplicationListInterface) => {
                setIsApplicationsAvailable(response.totalResults > 0);
            })
            .catch(() => {
                // Add debug logs here one a logger is added.
                // Tracked here https://github.com/wso2/product-is/issues/11650.
            })
            .finally(() => {
                setIsApplicationsFetchRequestLoading(false);
            });
    }, []);

    /**
     * Monitor `profileInfo.id` and publish the event to avoid an event without `UUID`.
     */
    useEffect(() => {

        if (!profileInfo?.id) {
            return;
        }

        // TODO: Move this to the `extensions/configs/common`.
        // Tracked here https://github.com/wso2-enterprise/asgardeo-product/issues/7742#issuecomment-939960128.
        eventPublisher.publish("console-click-getting-started-menu-item");
    }, [ profileInfo?.id ]);

    return (
        <PageLayout
            padded={ false }
            contentTopMargin={ false }
            data-testid={ `${ testId }-layout` }
            className="getting-started-page"
        >
            <div className="greeting">
                <Heading as="h1" bold="500" data-testid="welcome-greeting-header" ellipsis compact data-suppress="">
                    {
                        isProfileInfoLoading
                            ? (
                                <Placeholder data-testid="welcome-greeting-placeholder">
                                    <Placeholder.Header>
                                        <Placeholder.Line length="very long"/>
                                    </Placeholder.Header>
                                </Placeholder>
                            )
                            : t("extensions:common.quickStart.greeting.heading", {
                                username: resolveUserDisplayName(profileInfo)
                            })
                    }
                </Heading>
                <Heading as="h5" data-testid="welcome-greeting-sub-header" subHeading ellipsis>
                    {
                        t("extensions:common.quickStart.greeting.subHeading",
                            { productName: config.ui.productName })
                    }
                </Heading>
            </div>
            <Grid stackable stretched>
                <Show when={ AccessControlConstants.APPLICATION_READ }>
                    <Grid.Column mobile={ 16 } tablet={ 16 } computer={ 6 } largeScreen={ 6 } widescreen={ 6 }>
                        <Card
                            fluid
                            className="basic-card no-hover getting-started-card integrate-card"
                        >
                            <Card.Content className="illustration-container">
                                <GenericIcon
                                    transparent
                                    className="onboard-apps-animated-illustration"
                                    icon={ getGettingStartedCardIllustrations().onboardApplications }
                                />
                            </Card.Content>
                            <Card.Content extra className="description-container">
                                <div className="card-heading">
                                    <Heading as="h2">
                                        { t("extensions:common.quickStart.sections.integrateApps.heading") }
                                    </Heading>
                                </div>
                                <Text muted>
                                    { t("extensions:common.quickStart.sections.integrateApps.description") }
                                </Text>
                                <Label.Group>
                                    <Label>
                                        { t("extensions:common.quickStart.sections.integrateApps.capabilities.sso") }
                                    </Label>
                                    <Label>
                                        { t("extensions:common.quickStart.sections.integrateApps.capabilities.mfa") }
                                    </Label>
                                    <Label>
                                        {
                                            t("extensions:common.quickStart.sections.integrateApps.capabilities" +
                                                ".social")
                                        }
                                    </Label>
                                </Label.Group>
                            </Card.Content>
                            <Card.Content extra className="action-container">
                                <LinkButton
                                    compact
                                    loading={ isApplicationsFetchRequestLoading }
                                    data-testid="getting-started-page-add-app-button"
                                    className="quick-start-action"
                                    onClick={ () => {
                                        eventPublisher.publish("console-getting-started-add-application-path");

                                        history.push({
                                            pathname: isApplicationsAvailable
                                                ? AppConstants.getPaths().get("APPLICATIONS")
                                                : AppConstants.getPaths().get("APPLICATION_TEMPLATES")
                                        });
                                    } }
                                >
                                    <span className="button-text">
                                        <Show
                                            when={ AccessControlConstants.APPLICATION_WRITE }
                                            fallback={
                                                t("extensions:common.quickStart.sections.integrateApps.actions.view")
                                            }
                                        >
                                            {
                                                isApplicationsAvailable
                                                    ? t("extensions:common.quickStart.sections.integrateApps" +
                                                    ".actions.manage")
                                                    : t("extensions:common.quickStart.sections.integrateApps" +
                                                    ".actions.create")
                                            }
                                        </Show>
                                    </span>
                                </LinkButton>
                            </Card.Content>
                        </Card>
                    </Grid.Column>
                </Show>
                <Show when={ AccessControlConstants.USER_READ }>
                    <Grid.Column mobile={ 16 } tablet={ 16 } computer={ 6 } largeScreen={ 6 } widescreen={ 6 }>
                        <Card
                            fluid
                            className="basic-card no-hover getting-started-card manage-users-card"
                        >
                            <Card.Content className="illustration-container">
                                <GenericIcon
                                    transparent
                                    className="onboard-users-animated-illustration"
                                    icon={ getGettingStartedCardIllustrations().onboardUsers }
                                />
                            </Card.Content>
                            <Card.Content extra className="description-container">
                                <div className="card-heading">
                                    <Heading as="h2">
                                        { t("extensions:common.quickStart.sections.manageUsers.heading") }
                                    </Heading>
                                </div>
                                <Text muted>
                                    { t("extensions:common.quickStart.sections.manageUsers.description") }
                                </Text>
                                <Label.Group>
                                    <Label>
                                        {
                                            t("extensions:common.quickStart.sections.manageUsers" +
                                                ".capabilities.customers")
                                        }
                                    </Label>
                                    <Label>
                                        {
                                            t("extensions:common.quickStart.sections.manageUsers" +
                                                ".capabilities.collaborators")
                                        }
                                    </Label>
                                    <Label>
                                        {
                                            t("extensions:common.quickStart.sections.manageUsers" +
                                                ".capabilities.groups")
                                        }
                                    </Label>
                                </Label.Group>
                            </Card.Content>
                            <Card.Content className="action-container" extra>
                                <LinkButton
                                    compact
                                    loading={ isConsumerUsersFetchRequestLoading }
                                    data-testid="getting-started-page-add-user-button"
                                    className="quick-start-action"
                                    onClick={ () => {
                                        eventPublisher.publish("console-getting-started-add-users-path");
                                        history.push(AppConstants.getPaths().get("USERS"));
                                    } }
                                >
                                    <span className="button-text">
                                        <Show
                                            when={ AccessControlConstants.USER_WRITE }
                                            fallback={
                                                t("extensions:common.quickStart.sections.manageUsers.actions.view")
                                            }
                                        >
                                            {
                                                isConsumersAvailable
                                                    ? t("extensions:common.quickStart.sections.manageUsers.actions" +
                                                    ".manage")
                                                    : t("extensions:common.quickStart.sections.manageUsers.actions" +
                                                    ".create")
                                            }
                                        </Show>
                                    </span>
                                </LinkButton>
                            </Card.Content>
                        </Card>
                    </Grid.Column>
                </Show>
                <Grid.Column mobile={ 16 } tablet={ 16 } computer={ 4 } largeScreen={ 4 } widescreen={ 4 }>
                    <Show when={ AccessControlConstants.IDP_READ }>
                        <Card
                            fluid
                            className="basic-card no-hover getting-started-card social-connections-card"
                        >
                            <Card.Content className="illustration-container">
                                <GenericIcon
                                    transparent
                                    className="social-connections-animated-illustration"
                                    icon={ getGettingStartedCardIllustrations().setupSocialConnections }
                                />
                            </Card.Content>
                            <Card.Content className="description-container">
                                <div className="card-heading">
                                    <Heading as="h2">
                                        { t("extensions:common.quickStart.sections.addSocialLogin.heading") }
                                    </Heading>
                                </div>
                                <Text muted>
                                    { t("extensions:common.quickStart.sections.addSocialLogin.description") }
                                </Text>
                            </Card.Content>
                            <Card.Content extra className="action-container">
                                <LinkButton
                                    compact
                                    data-testid="develop-getting-started-page-add-social-login"
                                    className="quick-start-action"
                                    onClick={ () => {
                                        eventPublisher.publish("console-getting-started-add-social-connection-path");
                                        history.push({
                                            pathname: AppConstants.getPaths().get("CONNECTIONS")
                                        });
                                    } }
                                >
                                    <span className="button-text">
                                        <Show
                                            when={ AccessControlConstants.IDP_WRITE }
                                            fallback={
                                                t("extensions:common.quickStart.sections.addSocialLogin.actions.view")
                                            }
                                        >
                                            { t("extensions:common.quickStart.sections.addSocialLogin.actions.setup") }
                                        </Show>
                                    </span>
                                </LinkButton>
                            </Card.Content>
                        </Card>
                    </Show>
                    <Card
                        fluid
                        className="basic-card no-hover getting-started-card learn-card"
                    >
                        <Card.Content>
                            <div className="card-heading">
                                <Heading as="h2">
                                    { t("extensions:common.quickStart.sections.learn.heading") }
                                </Heading>
                            </div>
                        </Card.Content>
                        <Card.Content className="description-container">
                            <Text muted>
                                { t("extensions:common.quickStart.sections.learn.description") }
                            </Text>
                        </Card.Content>
                        <Card.Content extra className="action-container">
                            <LinkButton
                                compact
                                data-testid="develop-getting-started-page-view-docs"
                                className="quick-start-action"
                                onClick={ () => {
                                    eventPublisher.publish("console-getting-started-navigate-to-docs-path");
                                    window.open(docSiteURL, "_blank", "noopener");
                                } }
                            >
                                <span className="button-text">
                                    { t("extensions:common.quickStart.sections.learn.actions.view") }
                                </span>
                            </LinkButton>
                        </Card.Content>
                    </Card>
                </Grid.Column>
            </Grid>
            {
                !window[ "AppUtils" ].getConfig().extensions?.community && (
                    <>
                        <Divider hidden />
                        <div className="community-link-container">
                            <div className="contact-icon-container">
                                <GenericIcon
                                    transparent
                                    icon={ SupportHeadsetIcon }
                                    size="mini"
                                    fill="default"
                                    spaced="right"
                                />
                            </div>
                            <div>
                                <Heading as="h5" compact>Help & Support</Heading>
                                <Text compact muted>For assistance, contact
                                    { " " }
                                    <a href="mailto:asgardeo-help@wso2.com">asgardeo-help@wso2.com</a>
                                </Text>
                            </div>
                        </div>
                    </>
                )
            }
        </PageLayout>
    );
};

/**
 * Default props for the component.
 */
GettingStartedPage.defaultProps = {
    "data-testid": "getting-started-page"
};

/**
 * A default export was added to support React.lazy.
 * TODO: Change this to a named export once react starts supporting named exports for code splitting.
 * @see {@link https://reactjs.org/docs/code-splitting.html#reactlazy}
 */
export default GettingStartedPage;
